# -*- encoding: utf-8 -*-
# Module iasobel

from numpy import *
import numpy.oldnumeric.mlab as Mlab
from numpy.oldnumeric.mlab import *

def iasobel(f):
    from iapconv import iapconv
    from ia636 import iaimginfo

    def test_arctan2(x,y):
        from numpy import arctan2
        try:
            return (arctan2(x,y))
        except:
            return 0

    wx = array([[1.,2.,1.],
                [0.,0.,0.],
                [-1.,-2.,-1.]])
    wy = array([[1.,0.,-1.],
                [2.,0.,-2.],
                [1.,0.,-1.]])
    gx = iapconv(f, wx)
    gy = iapconv(f, wy)
    mag = abs(gx + gy*1j)
    theta = reshape(map(test_arctan2, ravel(gy), ravel(gx)), f.shape)
    return mag,theta




# -*- encoding: utf-8 -*-
# Module iaroi

from numpy import *

def iaroi(f, p1, p2):

    f = asarray(f)
    if len(f.shape) == 1: f = f[newaxis,:]

    g = f[p1[0]:p2[0]+1, p1[1]:p2[1]+1]
    return g




# -*- encoding: utf-8 -*-
# Module iageorigid

from numpy import *

def iageorigid(f, scale, theta, t):
    from iaffine import iaffine

    Ts   = [[scale[1],0,0], [0,scale[0],0], [0,0,1]]
    Trot = [[cos(theta),-sin(theta),0], [sin(theta),cos(theta),0], [0,0,1]]
    Tx   = [[1,0,t[1]], [0,1,t[0]], [0,0,1]]
    g = iaffine(f, dot(dot(Tx,Trot), Ts))
    return g




# -*- encoding: utf-8 -*-
# Module iadct

from numpy import *

def iadct(f):
    from iadctmatrix import iadctmatrix
    f = asarray(f).astype(float64)
    if len(f.shape) == 1: f = f[:,newaxis]
    (m, n) = f.shape
    if (n == 1):
        A = iadctmatrix(m)
        F = dot(A, f)
    else:
        A=iadctmatrix(m)
        B=iadctmatrix(n)
        F = dot(dot(A, f), transpose(B))
    return F




# -*- encoding: utf-8 -*-
# Module iaffine

import numpy as np

def iaffine(f,T,domain=0):

    if np.sum(domain) == 0:
        domain = f.shape
    if len(f.shape) == 2:
        H,W = f.shape
        y1,x1 = np.indices((domain))

        yx1 = np.array([ y1.ravel(),
                         x1.ravel(),
                         np.ones(np.product(domain))])
        yx_float = np.dot(np.linalg.inv(T), yx1)
        yy = np.rint(yx_float[0]).astype(int)
        xx = np.rint(yx_float[1]).astype(int)

        y = np.clip(yy,0,H-1).reshape(domain)
        x = np.clip(xx,0,W-1).reshape(domain)

        g = f[y,x]

    if len(f.shape) == 3:
        D,H,W = f.shape
        z1,y1,x1 = np.indices((domain))
        zyx1 = np.array([ z1.ravel(),
                          y1.ravel(),
                          x1.ravel(),
                          np.ones(np.product(domain)) ])
        zyx_float = np.dot(np.linalg.inv(T), zyx1)

        zz = np.rint(zyx_float[0]).astype(int)
        yy = np.rint(zyx_float[1]).astype(int)
        xx = np.rint(zyx_float[2]).astype(int)

        z = np.clip(zz, 0, D-1).reshape(domain) #z
        y = np.clip(yy, 0, H-1).reshape(domain) #rows
        x = np.clip(xx, 0, W-1).reshape(domain) #columns

        g = f[z,y,x]

    return g




# -*- encoding: utf-8 -*-
# Module iapca

import numpy as np

def iapca(X):

    n, dim = X.shape
    mu = X.mean(axis=0)
    X = X - mu

    C = np.dot(X.T,X)/(n-1)  # Covariance matrix
    e,V = np.linalg.eigh(C) # eigenvalues and eigenvectors of the covariance matrix
    indexes = np.argsort(e)[::-1] # sorting eigenvalues from largest
    e  = e [indexes]         # update e and V
    V = V[:,indexes]
    return V,e,mu




# -*- encoding: utf-8 -*-
# Module iaapplylut

from numpy import *

def iaapplylut(fi, it):
    g = it[fi]
    if len(g.shape) == 3:
        aux = zeros((3,g.shape[0],g.shape[1]), fi.dtype.name)
        for i in range(3):
            aux[i,:,:] = g[:,:,i]
        g = aux
    return g




# -*- encoding: utf-8 -*-
# Module iaffine3

def iaffine3(f, T, interpol='CLOSEST', destshape='SAME', destorg=(0,)):
    import numpy as np
    from ia636 import iainterpollin
    from ia636 import iainterpolclosest
    if T.ndim != 2 or T.shape[0] != T.shape[1]:
        raise ValueError, 'T must be a square matrix'

    dim = T.shape[0] - 1
    if dim != 2 and dim != 3:
        raise ValueError, 'T must be either 3x3 or 4x4 (2D or 3D transform)'

    if f.ndim < dim:
        raise ValueError, 'dimention of f is not compatible with the transform matrix'

    # if dimension of f is higher than dim then several planes need to be transformed.
    cor = (f.ndim > dim)

    # split the shape of f in two parts. sht is the part where the tranform is applied,
    # shh is not subject to the affine transform
    sht = f.shape[f.ndim - dim:]
    shh = f.shape[:f.ndim - dim]


    invT = np.linalg.inv(T.astype(np.float))
    # get the limits of the destination image
    shmin = np.asarray(destorg).astype(int)

    # f shape as a column
    fshapecol = np.asarray(sht)
    fshapecol.shape = (dim, 1)
    if destshape == 'SAME':
        shmax = np.asarray(sht)
        shmin = np.zeros((1), int)
    elif destshape == 'FIT':
        if dim == 2:
            M = np.asarray([[0,0,1,1],[0,1,0,1]]).astype(float)
        else:
            M =np.asarray([[0,0,0,0,1,1,1,1],[0,0,1,1,0,0,1,1],[0,1,0,1,0,1,0,1]]).astype(float)
        M *= (np.asarray(fshapecol)-1)
        M = np.vstack((M, np.ones((1,M.shape[1]))))
        M = np.dot(T, M)
        shmax = np.ceil(np.max(M, 1)[:-1]).astype(int)
        shmin = np.floor(np.min(M, 1)[:-1]).astype(int)
    else:
        shmin = np.asarray(destorg).astype(int)
        shmax = np.asarray(destshape).astype(int) + shmin
    # shmin in column shape
    shimincol = np.reshape(shmin, (shmin.shape[0], 1))

    # get indices in target space
    ind = np.indices(shmax-shmin)
    ind.shape = (ind.shape[0], np.prod(ind.shape[1:]))
    ind += shimincol

    # shape of target image
    dstshape = tuple(shmax - shmin)

    #apply inverse transform on the target coordinates
    tind = np.vstack((ind, np.ones((1, np.size(ind[0]))))).astype(float)
    oind = np.dot(invT, tind)

    # remove out of bounds indices
    oindr = oind.copy()
    oindr[:-1, :] = np.maximum(oind[:-1,:], 0)
    oindr[:-1, :] = np.minimum(oindr[:-1,:], fshapecol-1)

    # Select interpolation function
    if interpol=='LINEAR':
        interpolfunc = iainterpollin
    elif interpol=='CLOSEST':
        interpolfunc = iainterpolclosest
    else:
        # the user is providing his own interpolation function
        interpolfunc=interpol


    # interpolate data
    if cor:
        count = np.prod(shh)
        fr = np.reshape(f, ((count,) + sht))
        interpshape = ((count,) + (np.prod(dstshape),))
        g = np.empty(interpshape)
        for i in range(count):
            g[i] = interpolfunc(fr[i], oindr[0:-1,:])
            # set to 0 points out of bounds
            g[i, np.not_equal(np.sum(np.not_equal(oindr, oind), 0), 0)] = 0
        g.shape = (shh + dstshape)

    else:
        g = interpolfunc(f, oindr[0:-1,:])
        # set to 0 points out of bounds
        g[np.not_equal(np.sum(np.not_equal(oindr, oind), 0), 0)] = 0
        g.shape = dstshape
    return g




# -*- encoding: utf-8 -*-
# Module iafig2img

def iafig2img(figure):
    import numpy

    # draw the renderer
    figure.canvas.draw ( )

    # Get the RGB buffer from the figure
    w,h = figure.canvas.get_width_height()
    buf = numpy.fromstring ( figure.canvas.tostring_rgb(), dtype=numpy.uint8 )
    buf.shape = ( h, w, 3 )

    return buf.transpose((2,0,1))




# -*- encoding: utf-8 -*-
# Module iacontour

from numpy import *

def iacontour(f):

    f_ = f > 0
    new_shape = array(f_.shape)+2
    n = zeros(new_shape); n[0:-2,1:-1] = f_
    s = zeros(new_shape); s[2:: ,1:-1] = f_
    w = zeros(new_shape); w[1:-1,0:-2] = f_
    e = zeros(new_shape); e[1:-1,2:: ] = f_
    fi = logical_and(logical_and(logical_and(n,s),w),e)
    fi = fi[1:-1,1:-1]
    g = f_ - fi
    g = g > 0
    return g




# -*- encoding: utf-8 -*-
# Module iapconv

def iapconv4(f,h):
   import numpy as np

   h_ind=np.nonzero(h)
   f_ind=np.nonzero(f)
   if len(h_ind[0])>len(f_ind[0]):
       h,    f    = f,    h
       h_ind,f_ind= f_ind,h_ind

   gs = np.maximum(np.array(f.shape),np.array(h.shape))
   if (f.dtype == 'complex') or (h.dtype == 'complex'):
       g = np.zeros(gs,dtype='complex')
   else:
       g = np.zeros(gs)

   f1 = g.copy()
   f1[f_ind]=f[f_ind]

   if f.ndim == 1:
     (W,) = gs
     col = np.arange(W)
     for cc in h_ind[0]:
        g[:] += f1[(col-cc)%W] * h[cc]

   elif f.ndim == 2:
     H,W = gs
     row,col = np.indices(gs)
     for rr,cc in np.transpose(h_ind):
        g[:] += f1[(row-rr)%H, (col-cc)%W] * h[rr,cc]

   else:
     Z,H,W = gs
     d,row,col = np.indices(gs)
     for dd,rr,cc in np.transpose(h_ind):
        g[:] += f1[(d-dd)%Z, (row-rr)%H, (col-cc)%W] * h[dd,rr,cc]
   return g
from numpy import *

def iapconv(f, h):

    f, h = asarray(f), asarray(h,dtype=float)
    #faux, haux = ravel(f), ravel(h)

    s = f.shape
    # Checking the shape of the image F.
    if len(f.shape) == 1:
        f = f[newaxis,newaxis,:]
    elif len(f.shape) == 2:
        f = f[newaxis,:,:]

    # Checking the shape of the image H.
    if len(h.shape) == 1:
        h = h[newaxis,newaxis,:]
    elif len(h.shape) == 2:
        h = h[newaxis,:,:]

    # Getting the dimensions of images F and H
    (fslices, frows, fcols) = f.shape
    (hslices, hrows, hcols) = h.shape

    ds1 = int((hslices-1)/2.)
    ds2 = hslices - ds1

    dr1 = int((hrows-1)/2.)
    dr2 = hrows - dr1

    dc1 = int((hcols-1)/2.)
    dc2 = hcols - dc1

    p = concatenate((concatenate((f[-ds2+1::,:,:], f)), f[0:ds1,:,:]))
    p = concatenate((concatenate((p[:,-dr2+1::,:], p), 1), p[:,0:dr1,:]), 1)
    p = concatenate((concatenate((p[:,:,-dc2+1::], p), 2), p[:,:,0:dc1]), 2)

    g = zeros((fslices,frows,fcols))

    for i in range(hslices):
        for j in range(hrows):
            for k in range(hcols):
                hw = h[hslices-i-1, hrows-j-1, hcols-k-1]
                if (hw):
                    g = g + h[hslices-i-1,hrows-j-1,hcols-k-1] * p[i:fslices + i,j:frows + j, k:fcols + k]
    g.shape = s
    return g




# -*- encoding: utf-8 -*-
# Module ianeg

from numpy import *

def ianeg(f):

    f = asarray(f)
    if f.dtype in ['b','uint8','???']: # (numarray implementara uint16 e boolean)
        k = 2**(8*f.itemsize) - 1
        g = k - f
    else: # Trata os tipos com sinal
        g = -f.astype(float)
    return g




# -*- encoding: utf-8 -*-
# Module iarectangle

from numpy import *

def iarectangle(s, r, c):

    rows,  cols  = s[0], s[1]
    rrows, rcols = r[0], r[1]
    rr0,   cc0   = c[0], c[1]
    rr, cc = meshgrid(range(rows), range(cols), indexing='ij')

    min_row, max_row = rr0-rrows/2.0, rr0+rrows/2.0
    min_col, max_col = cc0-rcols/2.0, cc0+rcols/2.0

    g1 = (min_row <= rr) & (max_row > rr)
    g2 = (min_col <= cc) & (max_col > cc)

    g = g1 & g2
    return g




# -*- encoding: utf-8 -*-
# Module iah2stats

def iah2stats(h):
    import numpy as np
    import ia636 as ia

    hn = 1.0*h/h.sum() # compute the normalized image histogram
    v = np.zeros(11) # number of statistics

    # compute statistics
    n = len(h) # number of gray values
    v[0]  = np.sum((np.arange(n)*hn)) # mean
    v[1]  = np.sum(np.power((np.arange(n)-v[0]),2)*hn) # variance
    v[2]  = np.sum(np.power((np.arange(n)-v[0]),3)*hn)/(np.power(v[1],1.5))# skewness
    v[3]  = np.sum(np.power((np.arange(n)-v[0]),4)*hn)/(np.power(v[1],2))-3# kurtosis
    v[4]  = -(hn[hn>0]*np.log(hn[hn>0])).sum() # entropy
    v[5]  = np.argmax(h) # mode
    v[6:]  = ia.iah2percentile(h,np.array([1,10,50,90,99])) # 1,10,50,90,99% percentile
    return v




# -*- encoding: utf-8 -*-
# Module iatiling

import numpy as np
def iatiling(f, w):
  H = Hold = w
  g = np.zeros((H,w),'uint8')
  ih = iw = 0
  himax = 0
  for i in range(len(f)):
    hi,wi = f[i].shape
    if iw+wi > w:  # test if reach width
      iw = 0
      ih += himax + 1
      himax = 0
    if ih+hi > H:  # at any time test if reach height
      H += w
      g1 = np.zeros((H,w),'uint8')
      g1[:Hold,:] = g
      g = g1
      Hold = H
    g[ih:ih+hi, iw:iw+wi] = f[i] # insert another image
    iw += wi + 1
    himax = max(himax,hi)
  return g




# -*- encoding: utf-8 -*-
# Module iaellipse

from numpy import *
import math

def iaellipse(s, r, c, theta = 0):
    rows, cols = s[0], s[1]
    rr0,  cc0  = c[0], c[1]
    rr, cc = meshgrid(range(rows), range(cols), indexing='ij')
    rr = rr - rr0
    cc = cc - cc0
    cos = math.cos(theta)
    sen = math.sin(theta)
    i = cos/r[1]
    j = sen/r[0]
    m = -sen/r[1]
    n = cos/r[0]
    g = ((i*cc + m*rr)**2 + (j*cc + n*rr)**2) <= 1
    return g




# -*- encoding: utf-8 -*-
# Module iareadurl

def iareadurl(url):
    from StringIO import StringIO
    import urllib
    import PIL
    import adpil

    file = StringIO(urllib.urlopen(url).read())
    img = PIL.Image.open(file)
    return adpil.pil2array(img)




# -*- encoding: utf-8 -*-
# Module iahistogram

import numpy as np

def iahistogram(f):

    return np.bincount(f.ravel())
def iahistogram_eq(f):

    from numpy import amax, zeros, arange, sum

    n = amax(f) + 1
    h = zeros((n,),int)
    for i in arange(n):
      h[i] = sum(i == f)
    return h
def iahistogram_eq1(f):

    import numpy as np
    n = f.size
    m = f.max() + 1
    haux = np.zeros((m,n),int)
    fi = f.ravel()
    i = xrange(n)
    haux[fi,i] = 1
    h = haux.sum(axis=1)
    return h




# -*- encoding: utf-8 -*-
# Module iatile

from numpy import *

def iatile(f, new_size):

    f = asarray(f)
    if len(f.shape) == 1: f = f[newaxis,:]

    aux = resize(f, (new_size[0], f.shape[1]))
    aux = transpose(aux)
    aux = resize(aux, (new_size[1], new_size[0]))
    g = transpose(aux)
    return g




# -*- encoding: utf-8 -*-
# Module iarot90

from numpy import *

def iarot90(img, axis='X'):

   from ia636 import iaffine
   PIVAL = math.pi

   g = 0

   if axis == 'X':
      Trx = array([[cos(PIVAL/2), -sin(PIVAL/2), 0, img.shape[1] - 1],
                   [sin(PIVAL/2),  cos(PIVAL/2), 0, 0],
                   [           0,             0, 1, 0],
                   [           0,             0, 0, 1]]).astype(float)
      g = iaffine(img, Trx, [img.shape[1], img.shape[0], img.shape[2]])

   elif axis == 'Y':
      Try = array([[ cos(PIVAL/2), 0, sin(PIVAL/2), 0],
                    [            0, 1,            0, 0],
                    [-sin(PIVAL/2), 0, cos(PIVAL/2), img.shape[0] - 1],
                    [            0, 0,            0, 1]])
      g = iaffine(img, Try, [img.shape[2], img.shape[1], img.shape[0]])
   elif axis == 'Z':
      Trz = array([[1,            0,             0, 0],
                    [0, cos(PIVAL/2), -sin(PIVAL/2), img.shape[2] - 1],
                    [0, sin(PIVAL/2),  cos(PIVAL/2), 0],
                    [0,            0,             0, 1]]).astype(float)
      g = iaffine(img, Trz, [img.shape[0], img.shape[2], img.shape[1]])

   return g




# -*- encoding: utf-8 -*-
# Module iainterpolclosest

def iainterpolclosest(f, pts):
    # f - one, two or three dimention array
    # pts - array of points to interpolate
    import numpy as np
    ptsi = np.rint(pts).astype(int)
    ptsi[ptsi<0] = 0

    # make sure ptsi dimention is >= 2
    if ptsi.ndim==1:
        ptsi.shape = (1, ptsi.size)
    for i in range(0, f.ndim):
        ptsi[i] = np.minimum(ptsi[i], f.shape[i]-1)
    return f[list(ptsi)]




# -*- encoding: utf-8 -*-
# Module iacorr

def iacorr(im1, im2):
    import numpy as np
    avim1 = np.average(im1.astype(float).flat)
    avim2 = np.average(im2.astype(float).flat)

    im1x = im1.astype(float) - avim1
    im2x = im2.astype(float) - avim2

    cov = np.average(im1x*im2x)
    v1 = np.average(im1x*im1x)
    v2 = np.average(im2x*im2x)

    return cov/(np.sqrt(v1*v2))




# -*- encoding: utf-8 -*-
# Module ialogfilter

from numpy.fft import fft2, ifft2
from numpy import *

def ialogfilter(f, sigma):
    from ialog import ialog
    from iaifftshift import iaifftshift
    from iaisdftsym import iaisdftsym

    if len(shape(f)) == 1: f = f[newaxis,:]
    h = ialog(shape(f), map(int, array(shape(f))/2.), sigma)
    h = iaifftshift(h)
    H = fft2(h)
    if not iaisdftsym(H):
       raise Exception, "error: log filter is not symmetrical"
    G = fft2(f) * H
    g = ifft2(G).real
    return g




# -*- encoding: utf-8 -*-
# Module ialblshow

import numpy as np
from numpy.random import rand

def ialblshow(f):
    from iaapplylut import iaapplylut

    nblobs = f.max()
    r = np.floor(0.5 + 255*rand(nblobs, 1))
    g = np.floor(0.5 + 255*rand(nblobs, 1))
    b = np.floor(0.5 + 255*rand(nblobs, 1))
    ct = np.concatenate((r,g,b), 1)
    ct = np.concatenate(([[0,0,0]], ct))

    g = iaapplylut(f, ct)
    return g




# -*- encoding: utf-8 -*-
# Module iagaussian

import numpy as np

def iagaussian(s, mu, cov):
    d = len(s)  # dimension
    n = np.prod(s) # n. of samples (pixels)
    x = np.indices(s).reshape( (d, n))
    xc = x - mu
    k = 1. * xc * np.dot(np.linalg.inv(cov), xc)
    k = np.sum(k,axis=0) #the sum is only applied to the rows
    g = (1./((2 * np.pi)**(d/2.) * np.sqrt(np.linalg.det(cov)))) * np.exp(-1./2 * k)
    return g.reshape(s)




import os
from setuptools import setup

# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name="ia636",
    version="0.11",
    author="Roberto A Lotufo and collaborators",
    author_email="robertoalotufo@gmail.com",
    description=("Python Toolbox for Teaching Image Processing"),
    license="BSD 3 clauses",
    keywords="image processing",
    url="https://github.com/robertoalotufo/ia636",
    packages=['ia636'],
    long_description=read('README.txt'),
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Science/Research",
        "Programming Language :: Python :: 2 :: Only",
        "Topic :: Scientific/Engineering :: Image Recognition",
        "License :: OSI Approved :: BSD License",
    ],
)



# -*- encoding: utf-8 -*-
# Module ialog

from numpy import *

def ialog(s, mu, sigma):

    def test_exp(x,sigma):
       try:
          return (exp(-(x / (2. * sigma**2))))
       except:
          return 0

    mu = array(mu)
    if product(shape(s)) == 1:
       x = arange(s)
       r2 = (x-mu)**2
    else:
       (rr, cc) = indices( s)
       r2 = (rr-mu[0])**2  + (cc-mu[1])**2

    r2_aux = ravel(r2)
    aux = reshape(map(test_exp, r2_aux, 0*r2_aux+sigma), r2.shape)
    g = -(((r2 - 2 * sigma**2) / (sigma**4 * pi)) * aux)
    return g




# -*- encoding: utf-8 -*-
# Module iafloyd

from numpy import *

def iafloyd(f):
    from ianormalize import ianormalize

    f_ = 1.*ianormalize(f, [0,255])
    g = zeros(f_.shape)
    for i in range(f_.shape[0]):
        for j in range(f_.shape[1]):
            if f_[i,j] >= 128:
                g[i,j] = 255
            erro = f_[i,j] - g[i,j]
            if j < f_.shape[1]-1:
                f_[i,j+1] = f_[i,j+1] + 7*erro/16.
            if i < f_.shape[0]-1 and j > 0:
                f_[i+1,j-1] = f_[i+1,j-1] + 3*erro/16.
            if i < f_.shape[0]-1:
                f_[i+1,j] = f_[i+1,j] + 5*erro/16.
            if i < f_.shape[0]-1 and j < f_.shape[1]-1:
                f_[i+1,j+1] = f_[i+1,j+1] + erro/16.
    g = g > 0
    return g




# -*- encoding: utf-8 -*-
# Module iaptrans

from numpy import *

def iaptrans(f,t):
    import numpy as np
    g = np.empty(f.shape)
    if f.ndim == 1:
      W = f.shape[0]
      col = arange(W)
      g[:] = f[(col-t)%W]
    elif f.ndim == 2:
      H,W = f.shape
      rr,cc = t
      row,col = np.indices(f.shape)
      g[:] = f[(row-rr)%H, (col-cc)%W]
    elif f.ndim == 3:
      Z,H,W = f.shape
      zz,rr,cc = t
      z,row,col = np.indices(f.shape)
      g[:] = f[(z-zz)%Z, (row-rr)%H, (col-cc)%W]
    return g

# implementation using periodic convolution
def iaptrans2(f, t):
    from ia636 import iapconv

    f, t = asarray(f), asarray(t).astype(int32)
    h = zeros(2*abs(t) + 1)
    t = t + abs(t)
    h[tuple(t)] = 1
    g = iapconv(f, h)
    return g




# -*- encoding: utf-8 -*-
# Module ianshow

import ia636 as ia
import numpy as np

def t(s,dt=1):
   a = ia.iatext(s)
   b = np.pad(a,((6+3-dt,6+3-dt),(0+3-dt,0+3-dt)),'constant', constant_values=((False,False),(False,False)))
   c = np.pad(b,((dt,dt),(dt,dt)),'constant', constant_values=((True,True),(True,True)))
   return c

def timg(f,dt=1):

  tFalse = t('   ',dt)
  dy, dx = tFalse.shape
  tTrue = np.zeros_like(tFalse)
  z = np.empty(tuple(np.array(f.shape) * np.array([dy,dx]))).astype(bool)
  if f.dtype == 'bool':
    for x in np.arange(f.shape[-1]):
      for y in np.arange(f.shape[-2]):
        if f[y,x]:
          z[y*dy:y*dy+dy,x*dx:x*dx+dx] = tFalse
        else:
          z[y*dy:y*dy+dy,x*dx:x*dx+dx] = tTrue
    z=~np.pad(z,((1,1),(1,1)),'constant')
  else:
    for x in np.arange(f.shape[-1]):
      for y in np.arange(f.shape[-2]):
        z[y*dy:y*dy+dy,x*dx:x*dx+dx] = t('%3d' % f[y,x],dt)
    z=np.pad(~z,((1,1),(1,1)),'constant')
  return z

def ianshow(X, X1=None, X2=None, X3=None, X4=None, X5=None, X6=None):
  x = timg(X)
  x1,x2,x3,x4,x5,x6 = None,None,None,None,None,None
  if X1 is not None:
    x1 = ~timg(X1,3)
  if X2 is not None:
    x2 = ~timg(X2,3)
  if X3 is not None:
    x3 = ~timg(X3,3)
  if X4 is not None:
    x4 = ~timg(X4,3)
  if X5 is not None:
    x5 = ~timg(X5,3)
  if X6 is not None:
    x6 = ~timg(X6,3)
  return ia.iagshow(x,x1,x2,x3,x4,x5,x6)




# -*- encoding: utf-8 -*-
# Module iamosaic

from numpy import *
import scipy

def iamosaic(f,N,s=1.0):
    f = asarray(f)
    d,h,w = f.shape

    nLines = ceil(float(d)/N)
    nCells = nLines*N

    # Add black slices to match the exact number of mosaic cells
    fullf = resize(f, (nCells,h,w))
    fullf[d:nCells,:,:] = 0

    Y,X = indices((nLines*h,N*w))

    Pts = array([
            (floor(Y/h)*N + floor(X/w)).ravel(),
            mod(Y,h).ravel(),
            mod(X,w).ravel() ]).astype(int).reshape((3,nLines*h,N*w))

    g = scipy.ndimage.interpolation.zoom(fullf[Pts[0],Pts[1],Pts[2]],s,order=5)

    return g

